@@ -60,8 +60,8 @@ ZERO_OPTIMIZATION_OVERLAP_COMM_DEFAULT = False
 ZERO3_OPTIMIZATION_OVERLAP_COMM_DEFAULT = True
 
 ZERO_OPTIMIZATION_CONTIGUOUS_GRADIENTS = 'contiguous_gradients'
-ZERO_OPTIMIZATION_CONTIGUOUS_GRADIENTS_DEFAULT = False
-ZERO3_OPTIMIZATION_CONTIGUOUS_GRADIENTS_DEFAULT = False
+ZERO_OPTIMIZATION_CONTIGUOUS_GRADIENTS_DEFAULT = True
+ZERO3_OPTIMIZATION_CONTIGUOUS_GRADIENTS_DEFAULT = True
 
 ZERO_OPTIMIZATION_REDUCE_BUCKET_SIZE = 'reduce_bucket_size'
 ZERO_OPTIMIZATION_REDUCE_BUCKET_SIZE_DEFAULT = 500000000
